#![windows_subsystem = "windows"]

mod config;
mod ui;

use std::path::PathBuf;
use std::io::BufReader;
use std::fs::File;

use rodio::{Sink, Source};
use std::process::Command;

const SOUNDS_BUTTON_LABEL: &'static str = "open sounds folder";
const SPACING: i32 = 16;

#[cfg(target_os = "windows")]
fn file_manager() -> &'static str {
    "explorer"
}

#[cfg(not(target_os = "windows"))]
fn file_manager() -> &'static str {
    "xdg-open"
}

fn get_sounds_dir() -> PathBuf {
    let mut data_dir = dirs::data_dir().expect("Couldn't find user data directory");

    data_dir.push("ruin");
    data_dir.push("lull");

    // on windows, data dir and config dir are both User\AppData\Roaming
    #[cfg(target_os = "windows")] {
        data_dir.push("sounds");
    }

    if !data_dir.exists() {
        std::fs::create_dir_all(&data_dir)
            .expect("Couldn't create lull sounds directory");
    }

    data_dir
}

fn open_file_manager() {
    let mut file_manager = Command::new(file_manager());
    file_manager.arg(crate::get_sounds_dir());
    file_manager.output().unwrap();
}

struct Sound {
    name: String,
    sink: Sink,
    volume: f64,
}

fn get_sounds(device: &rodio::Device) -> Vec<Sound> {
    let mut sounds = Vec::new();

    let mut paths = std::fs::read_dir(get_sounds_dir())
        .expect("Couldn't read lull sounds directory")
        .map(|res| res.map(|e| e.path()))
        .collect::<Result<Vec<_>, std::io::Error>>()
        .expect("Couldn't read files from lull sounds directory");

    paths.sort();

    for path in paths {
        let name = path.file_stem().unwrap().to_str().unwrap().into();

        let file = File::open(&path)
            .expect("Couldn't open audio file");

        let source = rodio::Decoder::new(
            BufReader::new(file)
        );

        #[cfg(not(target_os = "windows"))]
        if source.is_err() {
            ui::gtk::error_popup(&format!(
                "Couldn't parse file {}. \n{}.",
                path.to_str().unwrap(),
                source.err().unwrap()
            ));
            continue;
        }

        let source = source.unwrap().repeat_infinite();

        let sink = rodio::Sink::new(&device);
        sink.append(source);
        sink.pause();

        sounds.push(Sound { name, sink, volume: 0.0 });
    }

    sounds
}

// #[cfg(not(target_os = "windows"))]
fn main() {
    crate::ui::gtk::instantiate();
}

// #[cfg(target_os = "windows")]
// fn main() {
//     eframe::run_native(Box::new(lull::ui::egui::State::default()));
// }

// fn main() {
//     crate::ui::orbtk::instantiate();
// }
