use std::env::args;

use gio::prelude::*;
use gtk::prelude::*;
use gtk::Orientation;

use crate::open_file_manager;
use crate::config::load_config;

pub(crate) fn error_popup(message: &str) {
    let popup = gtk::Window::new(gtk::WindowType::Toplevel);
    popup.set_title("error");
    popup.set_border_width(crate::SPACING as u32);
    popup.set_position(gtk::WindowPosition::Center);
    popup.set_default_size(256, 64);
    popup.set_type_hint(gdk::WindowTypeHint::Dialog);
    popup.set_resizable(false);

    let message = gtk::Label::new(Some(message));
    popup.add(&message);

    popup.show_all();
}

fn build_ui(application: &gtk::Application) {
    let window = gtk::ApplicationWindow::new(application);

    window.set_title("lull");
    window.set_border_width(crate::SPACING as u32);
    window.set_position(gtk::WindowPosition::Center);
    window.set_default_size(256, 128);

    if let Some(config) = load_config() {
        window.move_(config.position.0, config.position.1);
        window.resize(config.size.0, config.size.1);
    }

    let button_manage_sounds = gtk::Button::with_label(crate::SOUNDS_BUTTON_LABEL);

    button_manage_sounds.connect_clicked(|_| {
        open_file_manager();
    });

    let sounds_manage  = gtk::Box::new(Orientation::Vertical,   crate::SPACING);
    let columns        = gtk::Box::new(Orientation::Horizontal, crate::SPACING);
    let column_labels  = gtk::Box::new(Orientation::Vertical,   crate::SPACING);
    let column_sliders = gtk::Box::new(Orientation::Vertical,   crate::SPACING);

    columns.set_homogeneous(false);
    column_labels.set_homogeneous(true);
    column_sliders.set_homogeneous(true);
    column_labels.set_property_expand(false);
    column_sliders.set_property_expand(true);
    column_sliders.set_property_width_request(128);

    window.add(&sounds_manage);
    sounds_manage.add(&columns);
    sounds_manage.add(&button_manage_sounds);
    columns.add(&column_labels);
    columns.add(&column_sliders);

    let device = rodio::default_output_device().unwrap();

    let sounds = crate::get_sounds(&device);

    for sound in sounds {
        let label = gtk::Label::new(Some(sound.name.as_ref()));
        label.set_halign(gtk::Align::End);
        column_labels.add(&label);

        let adjustment = gtk::Adjustment::new(
            0.0,
            0.0,
            1.0,
            0.0,
            0.0,
            0.0
        );

        let slider = gtk::Scale::new(
            Orientation::Horizontal,
            Some(&adjustment)
        );

        slider.set_draw_value(false);

        slider.connect_value_changed(move |scale| {
            let volume = scale.get_value();

            if volume == 0. {
                sound.sink.pause();
            } else {
                sound.sink.play();
                sound.sink.set_volume(volume as f32);
            }
        });

        column_sliders.add(&slider);
    }

    window.show_all();

    window.connect_delete_event(|window, _event| {
        crate::config::save_config(crate::config::Config {
            position: window.get_position(),
            size:     window.get_size()
        });

        Inhibit(false)
    });
}

pub(crate) fn instantiate() {
    #[cfg(target_os = "windows")] {
        // instantiate rodio before gtk and don't do anything with it
        // this is silly but it's the easiest way to prevent cpal crashing on windows
        rodio::default_output_device().unwrap();
    }

    let application = gtk::Application::new(
        Some("dev.tinybird.max.lull"),
        Default::default()
    ).expect("Initialisation failed...");

    application.connect_activate(|app| {
        build_ui(app);
    });

    application.run(&args().collect::<Vec<_>>());
}
